@@ -711,7 +711,7 @@ GVApi::onLogin2(bool success, const QByteArray &response, QNetworkReply *reply,
 
         // After this we should have completed login. Check for cookie "gvx"
         foreach (QNetworkCookie cookie, jar->getAllCookies ()) {
-            if (cookie.name () == "gvx") {
+            if (cookie.name() == "gvx") {
                 loggedIn = true;
                 break;
             }
@@ -925,7 +925,7 @@ GVApi::getRnr(AsyncTaskToken *token)
 {
     Q_DEBUG("User authenticated, now looking for RNR.");
 
-    bool rv = doGet("https://www.google.com/voice/m/i/all", token, this,
+    bool rv = doGet(GV_HTTPS_M "/i/all", token, this,
                     SLOT (onGotRnr(bool, const QByteArray &,
                                    QNetworkReply *, void *)));
     Q_ASSERT(rv);
@@ -934,11 +934,12 @@ GVApi::getRnr(AsyncTaskToken *token)
 }//GVApi::getRnr
 
 void
-GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
+GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *reply,
                 void *ctx)
 {
     AsyncTaskToken *token = (AsyncTaskToken *)ctx;
     QString strResponse = response;
+    QString strReplyUrl = reply->url().toString();
 
     do { // Begin cleanup block (not a loop)
         if (!success) break;
@@ -946,6 +947,38 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
         success = false;
         int pos = strResponse.indexOf ("_rnr_se");
         if (pos == -1) {
+            if (token->outParams["attempts"].toInt() != 0) {
+                Q_WARN("Too many attempts at relogin");
+                break;
+            }
+
+            Q_DEBUG(QString("Current URL = %1. Attempting re-login")
+                        .arg(strReplyUrl));
+
+            // Probably failed to login correctly. Try one more time.
+            AsyncTaskToken *internalLogoutTask = new AsyncTaskToken(this);
+            if (!internalLogoutTask) {
+                Q_WARN("Failed to login because failed to logout!!");
+                break;
+            }
+
+            internalLogoutTask->callerCtx = token;
+
+            success =
+            connect(internalLogoutTask, SIGNAL(completed(AsyncTaskToken*)),
+                    this,  SLOT(internalLogoutForReLogin(AsyncTaskToken*)));
+            Q_ASSERT(success);
+
+            success = logout(internalLogoutTask);
+            if (!success) {
+                internalLogoutTask->deleteLater ();
+                break;
+            }
+
+            int attempts = 1;
+            token->outParams["attempts"] = attempts;
+
+            success = true;
             break;
         }
 
@@ -971,8 +1004,10 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
     } while (0); // End cleanup block (not a loop)
 
     if (!success) {
-        Q_WARN("Failed to get RNR. User cannot be authenticated.")
-                << strResponse;
+        QString msg = QString("Failed to get RNR. User cannot be "
+                              "authenticated. URL = %1. Response = %2")
+                        .arg(strReplyUrl, strResponse);
+        Q_WARN(msg);
 
         if (token) {
             token->status = ATTS_LOGIN_FAILURE;
@@ -981,6 +1016,14 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
     }
 }//GVApi::onGotRnr
 
+void
+GVApi::internalLogoutForReLogin(AsyncTaskToken *token)
+{
+    AsyncTaskToken *origToken = (AsyncTaskToken *) token->callerCtx;
+    login (origToken);
+    token->deleteLater ();
+}//GVApi::internalLogoutForReLogin
+
 bool
 GVApi::logout(AsyncTaskToken *token)
 {
@@ -1628,12 +1671,15 @@ GVApi::callOut(AsyncTaskToken *token)
         return true;
     }
 
+    QString dest = token->inParams["destination"].toString();
     QUrl url(GV_HTTPS_M "/x");
     url.addQueryItem("m" , "call");
-    url.addQueryItem("n" , token->inParams["destination"].toString());
+    url.addQueryItem("n" , dest);
     url.addQueryItem("f" , "");
     url.addQueryItem("v" , "7");
 
+    Q_DEBUG(QString("Call back: dest=%1, using=").arg(dest));
+
     QByteArray content;
     QList<QNetworkCookie> allCookies = jar->getAllCookies ();
     foreach (QNetworkCookie cookie, allCookies) {
@@ -1752,6 +1798,8 @@ GVApi::callBack(AsyncTaskToken *token)
     strContent += QString("&subscriberNumber=%1&remember=1&_rnr_se=%2")
                     .arg (strSelfNumber, rnr_se);
 
+    Q_DEBUG(QString("Call back request = %1").arg(strContent));
+
     bool rv = doPostForm(url, strContent.toAscii (), token, this,
                          SLOT(onCallback(bool, const QByteArray &,
                                          QNetworkReply *, void *)));
